<?php
namespace Util\Mail;
use Util\Mail\Core\SmtpMail;
use Util\Messages;
use Zend\Mime\Part as ZendMimePart;
use Zend\Mime\Message as ZendMimeMessage;
use Zend\Mail\Message;
use Zend\Mime\Message as MimeMessage;
use Zend\Mime\Mime;
use Zend\Mime\Part as MimePart;

class AttachmentMail {

    private $sender;
    private $smtpMail;
    private $baseDir = '';

    public function __construct(SmtpMail $smtpMail)
    {
        $this->sender   = "pm@teknisa.com";
        $this->smtpMail = $smtpMail;
        $this->baseDir = __DIR__.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'EmailTemplates'.DIRECTORY_SEPARATOR;
    }
    public function getLogMail()
    {
        return $this->baseDir . 'log_mail.html';
    }
    public function getLogMailTemplate()
    {
        $mainTemplate = $this->fileGetContentUtf8($this->getLogMail());
        return $mainTemplate;
    }
    function fileGetContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }
    
    private function encodeAndSend($to, $subject, $message)
    {           
        $message        = utf8_decode($message);
        $mimePart       = new ZendMimePart($message);
        $mimeMessage    = new ZendMimeMessage();
        $mimePart->type = 'text/html';
        $mimeMessage->setParts(array($mimePart));
        $this->sendHtml($to, $subject, $mimeMessage);
    }

    private function attachAndSend($email, $subject, $message, $filename, $dir)
    {
        $attach = new ZendMimePart(fopen($dir, 'r'));
        $attach->type = 'text/plain';
        $attach->filename = $filename;
        $attach->disposition = Mime::DISPOSITION_ATTACHMENT;
        $attach->encoding = Mime::ENCODING_BASE64;
        
        $commentTemplate = $this->buildEmailComment($message);
        $html = new MimePart($commentTemplate);
        $html->type = Mime::TYPE_HTML;
        $html->charset = 'utf-8';
        $html->encoding = Mime::ENCODING_QUOTEDPRINTABLE;

        $body = new MimeMessage();
        $body->setParts([$attach, $html]);

        $message = new Message();
        $message->setBody($body);

        $this->sendHtml($email, $subject, $body);        
    }

    public function buildEmailComment($comment)
    {
        $comment = $this->changeTagHtmlByEmail($comment);
        return str_replace("%comment%", $comment, $this->fileGetContentUtf8($this->getLogMail()));
    }
    private function changeTagHtmlByEmail($comment){
        $comment = str_replace("<", "&lt ", $comment);
        $comment = str_replace(">", " &gt", $comment);
        return $comment;
    }
    
    public function sendattachMails($email, $filename, $dir)
    {
        $this->attachAndSend($email, Messages::MAIL_SUBJECT_DEFAULT, Messages::MAIL_LOG_OPERATION , $filename, $dir);
    }

    public function sendIrregularitiesMail($email, $message)
    {
        $this->send($email, Messages::MAIL_LOG_OPERATION, $message);
    }

    private function send($to, $subject, $message)
    {
        $this->smtpMail->setFrom($this->sender);
        $this->smtpMail->setTo($to);
        $this->smtpMail->setSubject($subject);
        $this->smtpMail->setBody($message);
        $this->smtpMail->send();
    }

    private function sendHtml($to, $subject, $message)
    {
        $this->smtpMail->setFrom($this->sender);
        $this->smtpMail->setTo($to);
        $this->smtpMail->setSubject($subject);
        $this->smtpMail->setBody($message);
        $this->smtpMail->sendHtml();
    }

    
} 