<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 30/08/2016
 * Time: 14:28
 */

namespace Zeedhi\DevOps\Util;


use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\Session\Session;

class SocketClient {

    const NSP        = 'pm_do';
    const SERVER_KEY = 'you can create!';

    const WS_SERVER                    = 'wsServer';
    //Events response
    const RESPONSE_SUFFIX 			   = "response";
    const ERROR_SUFFIX 				   = "Error";
    // Events request
    const CREATE_NSP                   = 'createNamespace';
    const CREATE_ROOM                  = 'createRoom';
    const JOIN_ROOM                    = 'joinRoom';
    const REMOVE                       = 'remove';
    const NO_CLIENTS_CONNECTED         = 'noClientsConnected';
    const GET_PRIVATE_ROOM             = 'getPrivateRoom';

    protected $client;

    protected $timeout;

    public function __construct(\WebSocket\Client $client, $options) {
        $this->timeout = $options['timeout'];
        $this->client = $client;
        $this->connectToNamespace();
    }

    protected function send(array $payload) {
        $this->client->send(json_encode($payload));
    }

    protected function connectToNamespace() {
        $this->send($this->buildBasePayload(self::CREATE_NSP, null, array('key' => self::SERVER_KEY)));
    }

    /**
     * @return mixed
     */
    protected static function getWsServer()
    {
        $wsServer = InstanceManager::getInstance()->getParameter(self::WS_SERVER);
        return $wsServer;
    }

    protected function joinRoom($room) {
        $this->createRoom($room);
        $this->send($this->buildBasePayload(self::JOIN_ROOM, $room));
    }

    protected function createRoom($room) {
        $this->send($this->buildBasePayload(self::CREATE_ROOM, $room));
    }

    protected function buildBasePayload($event, $room = null, $data = array()) {
        return array(
            'event'     => $event,
            'room'      => $room,
            'namespace' => self::NSP,
            'data'      => $data
        );
    }
    protected static function buildDataRemove($path){
        return array('path' => $path);
    }

    public function removeDir($room,$path)
    {
        $this->joinRoom($room);
        $this->joinPrivateRoom($room);
        $dataRemove = self::buildDataRemove($path);
        $this->sendRemove($room, $dataRemove);
        $response = $this->waitForResponseSync(array(self::RESPONSE_SUFFIX));
        return $response['data'];
    }

    protected function waitForResponseSync(array $eventsToWait, $maxWaitTime = null, $stopOnFirstResponse = true) {
        //Sets a huge value of time limit to accept big timeouts
        set_time_limit(600);
        if ($maxWaitTime === null) {
            $maxWaitTime = $this->timeout;
        }
        $counter = 0;
        $response = array();

        $startTime = time();
        $shouldStopWaiting = false;
        while (time() < $startTime+$maxWaitTime && !$shouldStopWaiting) {
            try {
                $treatedRes = $this->treatResponsePayload($this->client->receive());
                if (empty($eventsToWait) || $this->checkSubscribedEvents($eventsToWait, $treatedRes['event'])) {
                    if ($stopOnFirstResponse) {
                        $shouldStopWaiting = true;
                        $response = $treatedRes;
                    }else{
                        $response[] = $treatedRes;
                    }

                }
            } catch(\WebSocket\ConnectionException $e) {
                //Any connection errors will be ignored
                //If all iterations are connection errors, the result will be a timeout
            } finally {
                $counter++;
            }
        }

        if (empty($response)) {
            $response[] = array (
                'error' => 'Maximum timeout reached',
                'data' => '',
                'hasError' => true
            );
        }

        return $response;
    }

    protected function treatResponsePayload($response) {
        return json_decode($response, true);
    }

    protected function checkSubscribedEvents($eventsToWait, $eventToCheck) {
        $result = false;
        foreach ($eventsToWait as $event) {
            if ($event === $eventToCheck) {
                $result = true;
            }
        }
        return $result;
    }

    public static function factory($options = array()) {
        if (empty($options['timeout'])) {
            $options['timeout'] = 4;
        }
        return new SocketClient(new \WebSocket\Client(self::getWsServer(), $options), $options);
    }

    /**
     * @param $room
     */
    protected function joinPrivateRoom($room)
    {
        $this->send($this->buildBasePayload(self::GET_PRIVATE_ROOM, $room, array('email' => $room)));
        $response = $this->waitForResponseSync(array(self::RESPONSE_SUFFIX));
        $privateEmail = $this->getPrivateEmail($response);
        $this->joinRoom($privateEmail);
    }

    /**
     * @param $response
     * @return string
     */
    protected function getPrivateEmail($response)
    {
        $privateEmail = '';
        if (empty($response['hasError'])) {
            $infoResponse = $response['data'];
            $privateEmail = $infoResponse['data'];
        }

        return $privateEmail;
    }

    /**
     * @param $room
     * @param $dataRemove
     */
    protected function sendRemove($room, $dataRemove)
    {
        $this->send($this->buildBasePayload(self::REMOVE, $room, $dataRemove));
    }
}